@@ -138,6 +138,7 @@ where
 {
     /// Create a new session from a low level stream
     pub fn new(raw_stream: T, config: Config, ty: SessionType) -> Session<T> {
+        assert!(config.max_stream_window_size >= crate::config::INITIAL_STREAM_WINDOW);
         let next_stream_id = match ty {
             SessionType::Client => 1,
             SessionType::Server => 2,
@@ -310,7 +311,6 @@ where
             frame_receiver,
             state,
             self.config.max_stream_window_size,
-            self.config.max_stream_window_size,
         );
         if let Err(err) = stream.send_window_update() {
             debug!("[{:?}] stream.send_window_update error={:?}", self.ty, err);
@@ -877,9 +877,16 @@ mod timer {
     }
 }
 
+#[cfg(test)]
+pub(crate) fn rt() -> &'static tokio::runtime::Runtime {
+    static RT: once_cell::sync::OnceCell<tokio::runtime::Runtime> =
+        once_cell::sync::OnceCell::new();
+    RT.get_or_init(|| tokio::runtime::Runtime::new().unwrap())
+}
+
 #[cfg(test)]
 mod test {
-    use super::Session;
+    use super::{rt, Session};
     use crate::{
         config::Config,
         frame::{Flag, Flags, Frame, FrameCodec, GoAwayCode, Type},
@@ -895,7 +902,7 @@ mod test {
         task::{Context, Poll},
         time::Duration,
     };
-    use tokio::io::{AsyncRead, AsyncReadExt, AsyncWrite, ReadBuf};
+    use tokio::io::{AsyncRead, AsyncReadExt, AsyncWrite, AsyncWriteExt, ReadBuf};
     use tokio_util::codec::Framed;
 
     struct MockSocket {
@@ -990,7 +997,7 @@ mod test {
 
     #[test]
     fn test_open_exist_stream() {
-        let rt = tokio::runtime::Runtime::new().unwrap();
+        let rt = rt();
 
         rt.block_on(async {
             let (remote, local) = MockSocket::new();
@@ -1048,7 +1055,7 @@ mod test {
     // the test will remain stuck and cannot be finished.
     #[test]
     fn test_close_session_on_stream_opened() {
-        let rt = tokio::runtime::Runtime::new().unwrap();
+        let rt = rt();
 
         rt.block_on(async {
             let (remote, local) = MockSocket::new();
@@ -1094,7 +1101,7 @@ mod test {
 
     #[test]
     fn test_open_too_many_stream() {
-        let rt = tokio::runtime::Runtime::new().unwrap();
+        let rt = rt();
 
         rt.block_on(async {
             let (remote, local) = MockSocket::new();
@@ -1146,7 +1153,7 @@ mod test {
 
     #[test]
     fn test_remote_does_not_respond_go_away() {
-        let rt = tokio::runtime::Runtime::new().unwrap();
+        let rt = rt();
 
         rt.block_on(async {
             let (_remote, local) = MockSocket::new();
@@ -1174,4 +1181,66 @@ mod test {
             }
         });
     }
+
+    #[test]
+    fn test_dynamically_config_the_window_size() {
+        let rt = rt();
+        rt.block_on(async {
+            let (remote, local) = MockSocket::new();
+
+            let config = Config::default();
+
+            let mut session = Session::new_server(local, config);
+
+            tokio::spawn(async move {
+                while let Some(Ok(mut stream)) = session.next().await {
+                    tokio::spawn(async move {
+                        let _ignore = stream.read_exact(&mut [0]).await;
+                        assert!(stream.send_window() == 1024 * 1024);
+                        assert!(stream.recv_window() == 256 * 1024 - 1);
+                        let mut buf = vec![1; 1024 * 1024];
+                        let _ignore = stream.write_all(&buf).await;
+                        let _ignore = stream.read(&mut buf).await;
+                    });
+                }
+            });
+
+            let config = Config {
+                max_stream_window_size: 1024 * 1024,
+                ..Default::default()
+            };
+
+            let mut client = Session::new_client(remote, config);
+
+            let mut control = client.control();
+
+            let mut stream = client.open_stream().unwrap();
+
+            tokio::spawn(async move {
+                loop {
+                    match client.next().await {
+                        Some(Ok(_)) => (),
+                        Some(Err(_)) => {
+                            break;
+                        }
+                        None => {
+                            break;
+                        }
+                    }
+                }
+            });
+
+            let _ignore = stream.write_all(&[1]).await;
+            assert!(stream.send_window() == 256 * 1024 - 1);
+            assert!(stream.recv_window() == 1024 * 1024);
+            let mut buf = vec![0; 1024 * 1024];
+            let _ignore = stream.read_exact(&mut buf).await;
+
+            tokio::spawn(async move {
+                control.close().await;
+            });
+
+            assert_eq!(vec![1; 1024 * 1024], buf)
+        })
+    }
 }
