@@ -18,6 +18,7 @@ use log::{debug, trace};
 use tokio::io::{AsyncRead, AsyncWrite, ReadBuf};
 
 use crate::{
+    config::INITIAL_STREAM_WINDOW,
     error::Error,
     frame::{Flag, Flags, Frame, Type},
     StreamId,
@@ -30,7 +31,7 @@ pub struct StreamHandle {
     state: StreamState,
 
     max_recv_window: u32,
-    recv_window: u32,
+    pub(crate) recv_window: u32,
     send_window: u32,
     read_buf: BytesMut,
 
@@ -52,16 +53,15 @@ impl StreamHandle {
         unbound_event_sender: UnboundedSender<StreamEvent>,
         frame_receiver: Receiver<Frame>,
         state: StreamState,
-        recv_window_size: u32,
-        send_window_size: u32,
+        max_window_size: u32,
     ) -> StreamHandle {
         assert!(state == StreamState::Init || state == StreamState::SynReceived);
         StreamHandle {
             id,
             state,
-            max_recv_window: recv_window_size,
-            recv_window: recv_window_size,
-            send_window: send_window_size,
+            max_recv_window: max_window_size,
+            recv_window: INITIAL_STREAM_WINDOW,
+            send_window: INITIAL_STREAM_WINDOW,
             read_buf: BytesMut::default(),
             unbound_event_sender,
             frame_receiver,
@@ -447,11 +447,9 @@ impl Drop for StreamHandle {
     fn drop(&mut self) {
         if !self.unbound_event_sender.is_closed() && self.state != StreamState::Closed {
             let event = StreamEvent::Closed(self.id);
-            if self.state == StreamState::LocalClosing {
-                // LocalClosing means that local have sent Fin to the remote and waiting for a response.
-                // So, here only need to send a cleanup message
-                let _ignore = self.unbound_event_sender.unbounded_send(event);
-            } else {
+            // LocalClosing means that local have sent Fin to the remote and waiting for a response.
+            // if not, we should send Rst first
+            if self.state != StreamState::LocalClosing {
                 let mut flags = self.get_flags();
                 flags.add(Flag::Rst);
                 let frame = Frame::new_window_update(flags, self.id, 0);
@@ -459,8 +457,8 @@ impl Drop for StreamHandle {
 
                 // Always successful unless the session is dropped
                 let _ignore = self.unbound_event_sender.unbounded_send(rst_event);
-                let _ignore = self.unbound_event_sender.unbounded_send(event);
             }
+            let _ignore = self.unbound_event_sender.unbounded_send(event);
         }
     }
 }
@@ -501,6 +499,7 @@ mod test {
     use crate::{
         config::INITIAL_STREAM_WINDOW,
         frame::{Flag, Flags, Frame, Type},
+        session::rt,
     };
     use bytes::BytesMut;
     use futures::{
@@ -512,7 +511,7 @@ mod test {
 
     #[test]
     fn test_drop() {
-        let rt = tokio::runtime::Runtime::new().unwrap();
+        let rt = rt();
         rt.block_on(async {
             let (_frame_sender, frame_receiver) = channel(2);
             let (unbound_sender, mut unbound_receiver) = unbounded();
@@ -522,7 +521,6 @@ mod test {
                 frame_receiver,
                 StreamState::Init,
                 INITIAL_STREAM_WINDOW,
-                INITIAL_STREAM_WINDOW,
             );
 
             drop(stream);
@@ -541,7 +539,7 @@ mod test {
 
     #[test]
     fn test_drop_with_state_reset() {
-        let rt = tokio::runtime::Runtime::new().unwrap();
+        let rt = rt();
         rt.block_on(async {
             let (mut frame_sender, frame_receiver) = channel(2);
             let (unbound_sender, mut unbound_receiver) = unbounded();
@@ -551,7 +549,6 @@ mod test {
                 frame_receiver,
                 StreamState::Init,
                 INITIAL_STREAM_WINDOW,
-                INITIAL_STREAM_WINDOW,
             );
 
             let mut flags = Flags::from(Flag::Syn);
@@ -577,7 +574,7 @@ mod test {
 
     #[test]
     fn test_drop_with_state_local_close() {
-        let rt = tokio::runtime::Runtime::new().unwrap();
+        let rt = rt();
         rt.block_on(async {
             let (_frame_sender, frame_receiver) = channel(2);
             let (unbound_sender, mut unbound_receiver) = unbounded();
@@ -587,7 +584,6 @@ mod test {
                 frame_receiver,
                 StreamState::Init,
                 INITIAL_STREAM_WINDOW,
-                INITIAL_STREAM_WINDOW,
             );
 
             let _ignore = stream.shutdown().await;
@@ -612,7 +608,7 @@ mod test {
 
     #[test]
     fn test_data_large_than_recv_window() {
-        let rt = tokio::runtime::Runtime::new().unwrap();
+        let rt = rt();
         rt.block_on(async {
             let (mut frame_sender, frame_receiver) = channel(2);
             let (unbound_sender, mut unbound_receiver) = unbounded();
@@ -621,10 +617,11 @@ mod test {
                 unbound_sender,
                 frame_receiver,
                 StreamState::Init,
-                2,
                 INITIAL_STREAM_WINDOW,
             );
 
+            stream.recv_window = 2;
+
             let flags = Flags::from(Flag::Syn);
             let frame = Frame::new_data(flags, 0, BytesMut::from("1234"));
             frame_sender.send(frame).await.unwrap();
@@ -654,7 +651,7 @@ mod test {
     // the protocol cannot be opened
     #[test]
     fn test_open_stream_with_data() {
-        let rt = tokio::runtime::Runtime::new().unwrap();
+        let rt = rt();
         rt.block_on(async {
             let (_frame_sender, frame_receiver) = channel(2);
             let (unbound_sender, mut unbound_receiver) = unbounded();
@@ -663,7 +660,6 @@ mod test {
                 unbound_sender,
                 frame_receiver,
                 StreamState::Init,
-                2,
                 INITIAL_STREAM_WINDOW,
             );
 
